﻿using Syncfusion.UI.Xaml.RichTextBoxAdv;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.System;
using Windows.UI.Core;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409

namespace Sample
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        #region Constructor
        /// <summary>
        /// Intializes a new instance of the <see cref="MainPage"/> class.
        /// </summary>
        public MainPage()
        {
            this.InitializeComponent();

            // Loads the document from stream.
            Assembly assembly = typeof(MainPage).GetTypeInfo().Assembly;
            Stream stream = assembly.GetManifestResourceStream("Sample.Assets.GettingStarted.docx");
            richTextBoxAdv.Load(stream, Syncfusion.UI.Xaml.RichTextBoxAdv.FormatType.Docx);
        }
        #endregion
    }

    #region Extension Class
    /// <summary>
    /// Provides an extension class for the <see cref="SfRichTextBoxAdv"/>.
    /// </summary>
    public class SfRichTextBoxAdvExtension : SfRichTextBoxAdv
    {
        #region Fields
        CoreWindow coreWindow;
        VirtualKeyModifiers ModifierKey;
        #endregion
        
        #region KeyDown Implementation
        /// <summary>
        /// Called before the KeyDown event occurs.
        /// </summary>
        /// <param name="e">The data for the event.</param>
        protected override void OnKeyDown(KeyRoutedEventArgs e)
        {
            if (coreWindow == null)
                coreWindow = CoreWindow.GetForCurrentThread();
            VirtualKeyModifiers modifier = GetKeyModifier(e.Key);
            if (modifier != VirtualKeyModifiers.None)
            {
                if (ModifierKey == VirtualKeyModifiers.None)
                    coreWindow.KeyUp += coreWindow_KeyUp;
                if (ModifierKey == VirtualKeyModifiers.None)
                    ModifierKey = modifier;
                else
                    ModifierKey |= modifier;
                base.OnKeyDown(e);
                return;
            }
            bool isHandled = false;
            if (ModifierKey == VirtualKeyModifiers.Control)
            {
                switch (e.Key)
                {
                    case VirtualKey.O:
                        // Disable CTRL + O Shortcut in SfRichTextBoxAdv control.
                        // Likewise you can disable any keyboard shortcut here.
                        isHandled = true;
                        break;
                }
            }
            if (isHandled)
                e.Handled = true;
            else
                base.OnKeyDown(e);
        }
        /// <summary>
        /// Gets the key modifier.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <returns>The Virtual key modifier.</returns>
        private VirtualKeyModifiers GetKeyModifier(VirtualKey key)
        {
            switch (key)
            {
                case VirtualKey.Application:
                    return VirtualKeyModifiers.Windows;
                case VirtualKey.Menu:
                    return VirtualKeyModifiers.Menu;
                case VirtualKey.Control:
                    return VirtualKeyModifiers.Control;
                case VirtualKey.Shift:
                    return VirtualKeyModifiers.Shift;
                default:
                    return VirtualKeyModifiers.None;
            }
        }
        /// <summary>
        /// Called when core window keyup.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args">An <see cref="T:Windows.UI.Core.KeyEventArgs">KeyEventArgs</see> that contains the event data.</param>
        void coreWindow_KeyUp(CoreWindow sender, KeyEventArgs args)
        {
            if (ModifierKey != VirtualKeyModifiers.None)
            {
                VirtualKeyModifiers modifier = GetKeyModifier(args.VirtualKey);
                if (modifier != VirtualKeyModifiers.None)
                    ModifierKey &= ~modifier;
            }
            if (ModifierKey == VirtualKeyModifiers.None)
            {
                if (coreWindow == null)
                    coreWindow = CoreWindow.GetForCurrentThread();
                coreWindow.KeyUp -= coreWindow_KeyUp;
            }
        }
        /// <summary>
        /// Called before the KeyUp event occurs.
        /// </summary>
        /// <param name="e">The data for the event.</param>
        protected override void OnKeyUp(KeyRoutedEventArgs e)
        {
            VirtualKeyModifiers modifier = GetKeyModifier(e.Key);
            if (modifier != VirtualKeyModifiers.None)
                ModifierKey &= ~modifier;
            if (ModifierKey != VirtualKeyModifiers.None)
            {
                if ((ModifierKey & VirtualKeyModifiers.Control) == VirtualKeyModifiers.Control
                    && coreWindow.GetKeyState(VirtualKey.Control) == CoreVirtualKeyStates.None)
                    ModifierKey &= ~VirtualKeyModifiers.Control;
                if ((ModifierKey & VirtualKeyModifiers.Menu) == VirtualKeyModifiers.Menu
                    && coreWindow.GetKeyState(VirtualKey.Menu) == CoreVirtualKeyStates.None)
                    ModifierKey &= ~VirtualKeyModifiers.Menu;
                if ((ModifierKey & VirtualKeyModifiers.Shift) == VirtualKeyModifiers.Shift
                    && coreWindow.GetKeyState(VirtualKey.Shift) == CoreVirtualKeyStates.None)
                    ModifierKey &= ~VirtualKeyModifiers.Shift;
                if ((ModifierKey & VirtualKeyModifiers.Windows) == VirtualKeyModifiers.Windows
                    && coreWindow.GetKeyState(VirtualKey.Application) == CoreVirtualKeyStates.None)
                    ModifierKey &= ~VirtualKeyModifiers.Windows;
            }
            base.OnKeyUp(e);
        }
        #endregion
    }
    #endregion
}
